@@ -138,8 +138,6 @@ njs_vm_create(njs_vm_opt_t *options)
         }
     }
 
-    vm->symbol_generator = NJS_SYMBOL_KNOWN_MAX;
-
     if (njs_scope_undefined_index(vm, 0) == NJS_INDEX_ERROR) {
         return NULL;
     }
@@ -222,7 +220,7 @@ njs_vm_compile(njs_vm_t *vm, u_char **start, u_char *end)
     global_items = (vm->global_scope != NULL) ? vm->global_scope->items : 0;
 
     ret = njs_parser_init(vm, &parser, vm->global_scope, &vm->options.file,
-                          *start, end, 0);
+                          *start, end);
     if (njs_slow_path(ret != NJS_OK)) {
         return NJS_ERROR;
     }
@@ -233,7 +231,7 @@ njs_vm_compile(njs_vm_t *vm, u_char **start, u_char *end)
     }
 
     if (njs_slow_path(vm->options.ast)) {
-        NJS_CHB_MP_INIT(&chain, vm);
+        NJS_CHB_MP_INIT(&chain, njs_vm_memory_pool(vm));
         ret = njs_parser_serialize_ast(parser.node, &chain);
         if (njs_slow_path(ret == NJS_ERROR)) {
             return ret;
@@ -330,7 +328,7 @@ njs_vm_compile_module(njs_vm_t *vm, njs_str_t *name, u_char **start,
         return NULL;
     }
 
-    ret = njs_parser_init(vm, &parser, NULL, name, *start, end, 1);
+    ret = njs_parser_init(vm, &parser, NULL, name, *start, end);
     if (njs_slow_path(ret != NJS_OK)) {
         return NULL;
     }
@@ -378,13 +376,24 @@ njs_vm_compile_module(njs_vm_t *vm, njs_str_t *name, u_char **start,
 }
 
 
+njs_int_t
+njs_vm_reuse(njs_vm_t *vm)
+{
+    vm->active_frame = NULL;
+    vm->top_frame = NULL;
+    vm->modules = NULL;
+
+    return njs_object_make_shared(vm, njs_object(&vm->global_value));
+}
+
+
 njs_vm_t *
 njs_vm_clone(njs_vm_t *vm, njs_external_ptr_t external)
 {
     njs_mp_t     *nmp;
     njs_vm_t     *nvm;
     njs_int_t    ret;
-    njs_value_t  **global;
+    njs_value_t  **global, **value;
 
     njs_thread_log_debug("CLONE:");
 
@@ -408,6 +417,11 @@ njs_vm_clone(njs_vm_t *vm, njs_external_ptr_t external)
     nvm->trace.data = nvm;
     nvm->external = external;
 
+    nvm->shared_atom_count = vm->atom_id_generator;
+
+    njs_lvlhsh_init(&nvm->atom_hash);
+    nvm->atom_hash_current = &nvm->atom_hash;
+
     ret = njs_vm_runtime_init(nvm);
     if (njs_slow_path(ret != NJS_OK)) {
         goto fail;
@@ -423,6 +437,24 @@ njs_vm_clone(njs_vm_t *vm, njs_external_ptr_t external)
         goto fail;
     }
 
+    if (nvm->options.unsafe) {
+        nvm->scope_absolute = njs_arr_create(nvm->mem_pool,
+                                             vm->scope_absolute->items,
+                                             sizeof(njs_value_t *));
+        if (njs_slow_path(nvm->scope_absolute == NULL)) {
+            goto fail;
+        }
+
+        value = njs_arr_add_multiple(nvm->scope_absolute,
+                                     vm->scope_absolute->items);
+        if (njs_slow_path(value == NULL)) {
+            goto fail;
+        }
+
+        memcpy(value, vm->scope_absolute->start,
+               vm->scope_absolute->items * sizeof(njs_value_t *));
+    }
+
     nvm->levels[NJS_LEVEL_GLOBAL] = global;
 
     /* globalThis and this */
@@ -446,25 +478,27 @@ njs_vm_runtime_init(njs_vm_t *vm)
     njs_int_t    ret;
     njs_frame_t  *frame;
 
-    frame = (njs_frame_t *) njs_function_frame_alloc(vm, NJS_FRAME_SIZE);
-    if (njs_slow_path(frame == NULL)) {
-        njs_memory_error(vm);
-        return NJS_ERROR;
+    if (vm->active_frame == NULL) {
+        frame = (njs_frame_t *) njs_function_frame_alloc(vm, NJS_FRAME_SIZE);
+        if (njs_slow_path(frame == NULL)) {
+            njs_memory_error(vm);
+            return NJS_ERROR;
+        }
+
+        frame->exception.catch = NULL;
+        frame->exception.next = NULL;
+        frame->previous_active_frame = NULL;
+
+        vm->active_frame = frame;
     }
 
-    frame->exception.catch = NULL;
-    frame->exception.next = NULL;
-    frame->previous_active_frame = NULL;
-
-    vm->active_frame = frame;
-
     ret = njs_regexp_init(vm);
     if (njs_slow_path(ret != NJS_OK)) {
         return NJS_ERROR;
     }
 
     njs_lvlhsh_init(&vm->values_hash);
-    njs_lvlhsh_init(&vm->keywords_hash);
+
     njs_lvlhsh_init(&vm->modules_hash);
 
     njs_rbtree_init(&vm->global_symbols, njs_symbol_rbtree_cmp);
@@ -843,12 +877,13 @@ njs_vm_value(njs_vm_t *vm, const njs_str_t *path, njs_value_t *retval)
             return NJS_ERROR;
         }
 
-        ret = njs_string_create(vm, &key, start, size);
+        ret = njs_atom_string_create(vm, &key, start, size);
         if (njs_slow_path(ret != NJS_OK)) {
             return NJS_ERROR;
         }
 
-        ret = njs_value_property(vm, &value, &key, njs_value_arg(retval));
+        ret = njs_value_property(vm, &value, key.atom_id,
+                                 njs_value_arg(retval));
         if (njs_slow_path(ret == NJS_ERROR)) {
             return ret;
         }
@@ -869,19 +904,20 @@ static njs_int_t
 njs_vm_bind2(njs_vm_t *vm, const njs_str_t *var_name, njs_object_prop_t *prop,
     njs_bool_t shared)
 {
-    njs_int_t           ret;
-    njs_object_t        *global;
-    njs_lvlhsh_t        *hash;
-    njs_lvlhsh_query_t  lhq;
+    njs_int_t            ret;
+    njs_value_t          prop_name;
+    njs_object_t         *global;
+    njs_flathsh_t        *hash;
+    njs_flathsh_query_t  lhq;
 
-    ret = njs_string_create(vm, &prop->name, var_name->start, var_name->length);
+    ret = njs_atom_string_create(vm, &prop_name, var_name->start,
+                                 var_name->length);
     if (njs_slow_path(ret != NJS_OK)) {
         return NJS_ERROR;
     }
 
     lhq.value = prop;
-    lhq.key = *var_name;
-    lhq.key_hash = njs_djb_hash(lhq.key.start, lhq.key.length);
+    lhq.key_hash = prop_name.atom_id;
     lhq.replace = 1;
     lhq.pool = vm->mem_pool;
     lhq.proto = &njs_object_hash_proto;
@@ -889,7 +925,7 @@ njs_vm_bind2(njs_vm_t *vm, const njs_str_t *var_name, njs_object_prop_t *prop,
     global = &vm->global_object;
     hash = shared ? &global->shared_hash : &global->hash;
 
-    ret = njs_lvlhsh_insert(hash, &lhq);
+    ret = njs_flathsh_unique_insert(hash, &lhq);
     if (njs_slow_path(ret != NJS_OK)) {
         njs_internal_error(vm, "lvlhsh insert failed");
         return ret;
@@ -905,7 +941,7 @@ njs_vm_bind(njs_vm_t *vm, const njs_str_t *var_name, const njs_value_t *value,
 {
     njs_object_prop_t   *prop;
 
-    prop = njs_object_prop_alloc(vm, &njs_value_undefined, value, 1);
+    prop = njs_object_prop_alloc(vm, value, 1);
     if (njs_slow_path(prop == NULL)) {
         return NJS_ERROR;
     }
@@ -921,8 +957,7 @@ njs_vm_bind_handler(njs_vm_t *vm, const njs_str_t *var_name,
 {
     njs_object_prop_t  *prop;
 
-    prop = njs_object_prop_alloc(vm, &njs_string_empty,
-                                 &njs_value_invalid, 1);
+    prop = njs_object_prop_alloc(vm, &njs_value_invalid, 1);
     if (njs_slow_path(prop == NULL)) {
         return NJS_ERROR;
     }
@@ -939,9 +974,9 @@ njs_vm_bind_handler(njs_vm_t *vm, const njs_str_t *var_name,
 
 
 void
-njs_value_string_get(njs_value_t *value, njs_str_t *dst)
+njs_value_string_get(njs_vm_t *vm, njs_value_t *value, njs_str_t *dst)
 {
-    njs_string_get(value, dst);
+    njs_string_get(vm, value, dst);
 }
 
 
@@ -1019,13 +1054,21 @@ njs_vm_prop_magic32(njs_object_prop_t *prop)
 
 
 njs_int_t
-njs_vm_prop_name(njs_vm_t *vm, njs_object_prop_t *prop, njs_str_t *dst)
+njs_vm_prop_name(njs_vm_t *vm, uint32_t atom_id, njs_str_t *dst)
 {
-    if (njs_slow_path(!njs_is_string(&prop->name))) {
+    njs_int_t    ret;
+    njs_value_t  prop_name;
+
+    ret = njs_atom_to_value(vm, &prop_name, atom_id);
+    if (ret != NJS_OK) {
         return NJS_ERROR;
     }
 
-    njs_string_get(&prop->name, dst);
+    if (njs_slow_path(!njs_is_string(&prop_name))) {
+        return NJS_ERROR;
+    }
+
+    njs_string_get(vm, &prop_name, dst);
 
     return NJS_OK;
 }
@@ -1059,7 +1102,7 @@ njs_vm_value_string(njs_vm_t *vm, njs_str_t *dst, njs_value_t *src)
                       && njs_number(src) == 0
                       && signbit(njs_number(src))))
     {
-        njs_string_get(&njs_string_minus_zero, dst);
+        *dst = njs_str_value("-0");
         return NJS_OK;
     }
 
@@ -1103,22 +1146,18 @@ njs_value_t *
 njs_vm_value_enumerate(njs_vm_t *vm, njs_value_t *value, uint32_t flags,
     njs_value_t *retval)
 {
-    njs_int_t                ret;
-    njs_value_t              *val;
-    njs_array_t              *keys;
-    njs_rbtree_t             *variables;
-    njs_rbtree_node_t        *rb_node;
-    njs_variable_node_t      *node;
-    const njs_lexer_entry_t  *lex_entry;
-
-    static const njs_str_t  njs_this_str = njs_str("this");
+    njs_value_t          *val;
+    njs_array_t          *keys;
+    njs_rbtree_t         *variables;
+    njs_rbtree_node_t    *rb_node;
+    njs_variable_node_t  *node;
 
     keys = njs_value_enumerate(vm, value, flags);
     if (njs_slow_path(keys == NULL)) {
         return NULL;
     }
 
-    if (!njs_values_same(value, &vm->global_value)
+    if (!njs_values_same(vm, value, &vm->global_value)
         || vm->global_scope == NULL)
     {
         goto done;
@@ -1132,12 +1171,7 @@ njs_vm_value_enumerate(njs_vm_t *vm, njs_value_t *value, uint32_t flags,
     while (njs_rbtree_is_there_successor(variables, rb_node)) {
         node = (njs_variable_node_t *) rb_node;
 
-        lex_entry = njs_lexer_entry(node->variable->unique_id);
-        if (njs_slow_path(lex_entry == NULL)) {
-            return NULL;
-        }
-
-        if (njs_strstr_eq(&lex_entry->name, &njs_this_str)) {
+        if (node->variable->atom_id == NJS_ATOM_STRING_this) {
             rb_node = njs_rbtree_node_successor(variables, rb_node);
             continue;
         }
@@ -1147,11 +1181,7 @@ njs_vm_value_enumerate(njs_vm_t *vm, njs_value_t *value, uint32_t flags,
             return NULL;
         }
 
-        ret = njs_string_create(vm, val, lex_entry->name.start,
-                                lex_entry->name.length);
-        if (njs_slow_path(ret != NJS_OK)) {
-            return NULL;
-        }
+        njs_atom_to_value(vm, val, node->variable->atom_id);
 
         rb_node = njs_rbtree_node_successor(variables, rb_node);
     }
@@ -1184,12 +1214,12 @@ njs_vm_value_own_enumerate(njs_vm_t *vm, njs_value_t *value, uint32_t flags,
 njs_int_t
 njs_vm_object_alloc(njs_vm_t *vm, njs_value_t *retval, ...)
 {
-    va_list             args;
-    njs_int_t           ret;
-    njs_value_t         *name, *value;
-    njs_object_t        *object;
-    njs_object_prop_t   *prop;
-    njs_lvlhsh_query_t  lhq;
+    va_list              args;
+    njs_int_t            ret;
+    njs_value_t          *name, *value;
+    njs_object_t         *object;
+    njs_object_prop_t    *prop;
+    njs_flathsh_query_t  lhq;
 
     object = njs_object_alloc(vm);
     if (njs_slow_path(object == NULL)) {
@@ -1217,21 +1247,25 @@ njs_vm_object_alloc(njs_vm_t *vm, njs_value_t *retval, ...)
             goto done;
         }
 
-        lhq.replace = 0;
-        lhq.pool = vm->mem_pool;
-        lhq.proto = &njs_object_hash_proto;
-
-        njs_string_get(name, &lhq.key);
-        lhq.key_hash = njs_djb_hash(lhq.key.start, lhq.key.length);
-
-        prop = njs_object_prop_alloc(vm, name, value, 1);
+        prop = njs_object_prop_alloc(vm, value, 1);
         if (njs_slow_path(prop == NULL)) {
             goto done;
         }
 
-        lhq.value = prop;
+        if (name->atom_id == NJS_ATOM_STRING_unknown) {
+            ret = njs_atom_atomize_key(vm, name);
+            if (ret != NJS_OK) {
+                goto done;
+            }
+        }
 
-        ret = njs_lvlhsh_insert(&object->hash, &lhq);
+        lhq.value = prop;
+        lhq.key_hash = name->atom_id;
+        lhq.replace = 0;
+        lhq.pool = vm->mem_pool;
+        lhq.proto = &njs_object_hash_proto;
+
+        ret = njs_flathsh_unique_insert(&object->hash, &lhq);
         if (njs_slow_path(ret != NJS_OK)) {
             njs_internal_error(vm, NULL);
             goto done;
@@ -1326,7 +1360,7 @@ njs_vm_object_prop(njs_vm_t *vm, njs_value_t *value, const njs_str_t *prop,
         return NULL;
     }
 
-    ret = njs_value_property(vm, value, &key, njs_value_arg(retval));
+    ret = njs_value_property_val(vm, value, &key, njs_value_arg(retval));
     if (njs_slow_path(ret != NJS_OK)) {
         return NULL;
     }
@@ -1347,12 +1381,13 @@ njs_vm_object_prop_set(njs_vm_t *vm, njs_value_t *value, const njs_str_t *prop,
         return NJS_ERROR;
     }
 
-    ret = njs_vm_value_string_create(vm, &key, prop->start, prop->length);
+    ret = njs_atom_string_create(vm, &key, prop->start, prop->length);
     if (njs_slow_path(ret != NJS_OK)) {
         return NJS_ERROR;
     }
 
-    ret = njs_value_property_set(vm, value, &key, njs_value_arg(setval));
+    ret = njs_value_property_set(vm, value, key.atom_id,
+                                 njs_value_arg(setval));
     if (njs_slow_path(ret != NJS_OK)) {
         return NJS_ERROR;
     }
@@ -1442,8 +1477,6 @@ njs_vm_date_alloc(njs_vm_t *vm, njs_value_t *retval, double time)
 njs_int_t
 njs_vm_value_to_string(njs_vm_t *vm, njs_str_t *dst, njs_value_t *src)
 {
-    u_char       *start;
-    size_t       size;
     njs_int_t    ret;
     njs_value_t  value, stack;
 
@@ -1453,7 +1486,7 @@ njs_vm_value_to_string(njs_vm_t *vm, njs_str_t *dst, njs_value_t *src)
 
     if (njs_is_error(src)) {
         if (njs_is_memory_error(vm, src)) {
-            njs_string_get(&njs_string_memory_error, dst);
+            *dst = njs_str_value("MemoryError");
             return NJS_OK;
         }
 
@@ -1472,24 +1505,7 @@ njs_vm_value_to_string(njs_vm_t *vm, njs_str_t *dst, njs_value_t *src)
     ret = njs_value_to_string(vm, &value, &value);
 
     if (njs_fast_path(ret == NJS_OK)) {
-        size = value.short_string.size;
-
-        if (size != NJS_STRING_LONG) {
-            start = njs_mp_alloc(vm->mem_pool, size);
-            if (njs_slow_path(start == NULL)) {
-                njs_memory_error(vm);
-                return NJS_ERROR;
-            }
-
-            memcpy(start, value.short_string.start, size);
-
-        } else {
-            size = value.long_string.size;
-            start = value.long_string.data->start;
-        }
-
-        dst->length = size;
-        dst->start = start;
+        njs_string_get(vm, &value, dst);
     }
 
     return ret;
@@ -1504,24 +1520,12 @@ njs_vm_value_to_string(njs_vm_t *vm, njs_str_t *dst, njs_value_t *src)
 const char *
 njs_vm_value_to_c_string(njs_vm_t *vm, njs_value_t *value)
 {
-    u_char  *p, *data, *start;
+    u_char  *p, *data;
     size_t  size;
 
     njs_assert(njs_is_string(value));
 
-    if (value->short_string.size != NJS_STRING_LONG) {
-        start = value->short_string.start;
-        size = value->short_string.size;
-
-        if (size < NJS_STRING_SHORT) {
-            start[size] = '\0';
-            return (const char *) start;
-        }
-
-    } else {
-        start = value->long_string.data->start;
-        size = value->long_string.size;
-    }
+    size = value->string.data->size;
 
     data = njs_mp_alloc(vm->mem_pool, size + njs_length("\0"));
     if (njs_slow_path(data == NULL)) {
@@ -1529,7 +1533,7 @@ njs_vm_value_to_c_string(njs_vm_t *vm, njs_value_t *value)
         return NULL;
     }
 
-    p = njs_cpymem(data, start, size);
+    p = njs_cpymem(data, value->string.data->start, size);
     *p++ = '\0';
 
     return (const char *) data;
@@ -1564,8 +1568,7 @@ njs_value_to_string(njs_vm_t *vm, njs_value_t *dst, njs_value_t *value)
 njs_int_t
 njs_vm_value_to_bytes(njs_vm_t *vm, njs_str_t *dst, njs_value_t *src)
 {
-    u_char              *start;
-    size_t              size, length, offset;
+    size_t              length, offset;
     njs_int_t           ret;
     njs_value_t         value;
     njs_typed_array_t   *array;
@@ -1616,24 +1619,8 @@ njs_vm_value_to_bytes(njs_vm_t *vm, njs_str_t *dst, njs_value_t *src)
             return NJS_ERROR;
         }
 
-        size = value.short_string.size;
-
-        if (size != NJS_STRING_LONG) {
-            start = njs_mp_alloc(vm->mem_pool, size);
-            if (njs_slow_path(start == NULL)) {
-                njs_memory_error(vm);
-                return NJS_ERROR;
-            }
-
-            memcpy(start, value.short_string.start, size);
-
-        } else {
-            size = value.long_string.size;
-            start = value.long_string.data->start;
-        }
-
-        dst->length = size;
-        dst->start = start;
+        dst->length = value.string.data->size;
+        dst->start = value.string.data->start;
     }
 
     return ret;
@@ -1641,9 +1628,10 @@ njs_vm_value_to_bytes(njs_vm_t *vm, njs_str_t *dst, njs_value_t *src)
 
 
 njs_int_t
-njs_vm_string_compare(const njs_value_t *v1, const njs_value_t *v2)
+njs_vm_string_compare(njs_vm_t *vm, const njs_value_t *v1,
+    const njs_value_t *v2)
 {
-    return njs_string_cmp(v1, v2);
+    return njs_string_cmp(vm, v1, v2);
 }
 
 
